/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.opengles.GL11;

import android.opengl.GLU;

public class Camera extends Actor {
	public Camera(Target aTarget) {
		super();

		target = aTarget;
		basePosition = new Vector();
		targetOffset = Vector.getRandomizedInCube(-192.0f, 192.0f, -192.0f, 192.0f, -192.0f, 192.0f);
		offset = new Vector();
		viewTargetPosition = new Vector();
		count = 240;
	}

	@Override
	public void process() {
		Vector previousPosition = new Vector();
		previousPosition.set(position);
		count--;
		if (count <= 0) {
			targetOffset = Vector.getRandomizedInCube(-192.0f, 192.0f, -192.0f, 192.0f, -192.0f, 192.0f);
			count = 240;
		}
		Vector targetPosition = target.getPosition();
		viewTargetPosition.setX(viewTargetPosition.getX() + (targetPosition.getX() - viewTargetPosition.getX()) * 0.06f);
		viewTargetPosition.setY(viewTargetPosition.getY() + (targetPosition.getY() - viewTargetPosition.getY()) * 0.06f);
		viewTargetPosition.setZ(viewTargetPosition.getZ() + (targetPosition.getZ() - viewTargetPosition.getZ()) * 0.06f);
		Vector difference = targetPosition.getSubtracted(basePosition);
		basePosition.setX(basePosition.getX() + difference.getX() * 0.05f);
		basePosition.setY(basePosition.getY() + difference.getY() * 0.05f);
		basePosition.setZ(basePosition.getZ() + difference.getZ() * 0.05f);

		difference = targetOffset.getSubtracted(offset);
		if (difference.getLength() > 1.0f) {
			difference.normalize();
		}
		offset.add(difference);

		position.set(basePosition.getAdded(offset));
		velocity.set(position.getSubtracted(previousPosition));

		Vector viewVector = viewTargetPosition.getSubtracted(position);
		viewVector.normalize();
		Vector xVector = new Vector(1.0f, 0.0f, 0.0f);
		Vector viewUpVector = viewVector.getCrossProduct(xVector);
		viewUpVector.normalize();
		Vector viewXVector = viewUpVector.getCrossProduct(viewVector);
		viewXVector.normalize();
		direction.setAt(0 * 4 + 0, viewXVector.getX());
		direction.setAt(0 * 4 + 1, viewXVector.getY());
		direction.setAt(0 * 4 + 2, viewXVector.getZ());
		direction.setAt(1 * 4 + 0, viewUpVector.getX());
		direction.setAt(1 * 4 + 1, viewUpVector.getY());
		direction.setAt(1 * 4 + 2, viewUpVector.getZ());
		direction.setAt(2 * 4 + 0, viewVector.getX());
		direction.setAt(2 * 4 + 1, viewVector.getY());
		direction.setAt(2 * 4 + 2, viewVector.getZ());
	}

	@Override
	public void draw(GL11 gl) {
		GLU.gluLookAt(gl, position.getX(), position.getY(), position.getZ(), viewTargetPosition.getX(), viewTargetPosition.getY(), viewTargetPosition.getZ(), 0.0f, 1.0f, 0.0f);
	}

	public Matrix getDirection() {
		return direction;
	}

	private Target target;
	private Vector basePosition;
	private Vector targetOffset;
	private Vector offset;
	private Vector viewTargetPosition;
	private int count;
}
