/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import javax.microedition.khronos.opengles.GL11;

import android.os.SystemClock;

class DancingMissilesRenderer implements GLWallpaperService.Renderer {
	public DancingMissilesRenderer() {
		firstBegunTime = SystemClock.uptimeMillis();
		lastBegunTime = firstBegunTime;
		lastFrameNum = 0;
		frameRateCount = 0;
		drawingCount = 0;
	}

	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		// Do nothing
	}

	public void onSurfaceChanged(GL10 gl, int width, int height) {
		gl.glViewport(0, 0, width, height);
		scene.setSurfaceSize(width, height);
		scene.invalidateTextues((GL11)gl);
	}

	public void onDrawFrame(GL10 gl) {
		long begunTime = SystemClock.uptimeMillis();
		long difference = begunTime - firstBegunTime;
		if (difference < 0) {
			firstBegunTime = begunTime;
			lastBegunTime = firstBegunTime;
			difference = 1;
			lastFrameNum = 0;
		}
		long frameNum = difference / 16;
		int processingCount = (int)(frameNum - lastFrameNum);
		int sleepingTime = 16 - ((int)(begunTime - lastBegunTime) - processingCount * 16);
		if (processingCount == 0) {
			processingCount++;
			try {
				Thread.sleep(sleepingTime, 0);
			}
			catch (InterruptedException exception) {
			}
		}
		if (processingCount > 60) {
			processingCount = 60;
		}
		lastBegunTime = begunTime;
		lastFrameNum = frameNum;

		// Wait for screen shot
		/*
		try {
			Thread.sleep(1000);
		}
		catch (InterruptedException exception) {
		}
		*/

		for (int i = 0; i < processingCount; i++) {
			scene.process();
		}

		scene.draw((GL11)gl, 0.0f);

		drawingCount++;
		frameRateCount += processingCount;
		if (frameRateCount >= 60) {
			scene.setFps((float)drawingCount / (float)frameRateCount * 60.0f);
			frameRateCount = 0;
			drawingCount = 0;
		}
	}

	public void setScene(Scene aScene) {
		scene = aScene;
	}

	private Scene scene;
	private long firstBegunTime;
	private long lastBegunTime;
	private long lastFrameNum;
	private int frameRateCount;
	private int drawingCount;
}
