/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.lang.ref.WeakReference;
import java.util.ArrayList;

import android.content.Intent;
import android.content.SharedPreferences;
import android.view.SurfaceHolder;

public class DancingMissilesService extends GLWallpaperService implements SharedPreferences.OnSharedPreferenceChangeListener {
	@Override
	public void onCreate() {
		super.onCreate();
		engines = new ArrayList<WeakReference<DancingMissilesEngine>>();
		preferences = getSharedPreferences(SHARED_PREFERENCES_NAME, 0);
		preferences.registerOnSharedPreferenceChangeListener(this);
		engineNum = 0;
		startId = 0;
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int aStartId) {
		int result = super.onStartCommand(intent, flags, aStartId);
		startId = aStartId;

		return result;
	}

	@Override
	public Engine onCreateEngine() {
		DancingMissilesEngine engine = new DancingMissilesEngine(this, preferences);
		engines.add(new WeakReference<DancingMissilesEngine>(engine));
		engineNum++;
		return engine;
	}

	public void onDestroyEngine() {
		cleanupEngines();
		engineNum--;
		if (engineNum <= 0) {
			stopSelf(startId);
		}
	}

	public void onSharedPreferenceChanged(SharedPreferences arg0, String arg1) {
		Settings settings = Settings.getInstance();
		settings.update(preferences);

		cleanupEngines();
		for (WeakReference<DancingMissilesEngine> engine: engines) {
			if (engine.get() != null) {
				engine.get().onSharedPreferenceChanged();
			}
		}
	}

	private void cleanupEngines() {
		ArrayList<WeakReference<DancingMissilesEngine>> newEngines = new ArrayList<WeakReference<DancingMissilesEngine>>();
		for (WeakReference<DancingMissilesEngine> engine: engines) {
			if (engine.get() != null) {
				newEngines.add(engine);
			}
		}
		engines = newEngines;
	}
	
	class DancingMissilesEngine extends GLEngine {
		DancingMissilesEngine(DancingMissilesService aService, SharedPreferences aPreferences) {
			super();
			service = aService;
			renderer = new DancingMissilesRenderer();
			preferences = aPreferences;
			Settings settings = Settings.getInstance();
			settings.update(preferences);
			scene = new DancingMissilesScene(service);
			setRenderer(renderer);
			setRenderMode(RENDERMODE_CONTINUOUSLY);
			renderer.setScene(scene);
		}

		@Override
		public void onCreate(SurfaceHolder surfaceHolder) {
			super.onCreate(surfaceHolder);
		}

		@Override
		public void onDestroy() {
			service.onDestroyEngine();
			super.onDestroy();
		}

		public void onSharedPreferenceChanged() {
			scene = new DancingMissilesScene(service);
			renderer.setScene(scene);
		}

		private DancingMissilesService service;
		private SharedPreferences preferences;
		private DancingMissilesScene scene;
		private DancingMissilesRenderer renderer;
	}

	public static final String SHARED_PREFERENCES_NAME = "dancingmissilessettings";

	private ArrayList<WeakReference<DancingMissilesEngine>> engines;
	private SharedPreferences preferences;
	private int engineNum;
	private int startId;
}
