/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.opengles.GL11;

public class Explosion extends Actor {
	public Explosion(Vector aPosition, Vector aVelocity, float aScale, int aStep, DancingMissilesScene aScene) {
		super();

		position.set(aPosition);
		velocity.set(aVelocity);
		scene = aScene;
		scale = aScale;
		count = 0;
		step = aStep;
	}

	@Override
	public void process() {
		super.process();
		count += step;
		if (count >= 256) {
			scene.getExplosions().remove(this);			
		}
	}

	@Override
	public void draw(GL11 gl) {
		gl.glPushMatrix();
		gl.glTranslatef(position.getX(), position.getY(), position.getZ());
		gl.glScalef(scale, scale, scale);
		gl.glMultMatrixf(scene.getCamera().getDirection().getValues(), 0);
		float alpha = 1.0f;
		if (count >= 128)
		{
			alpha = 1.0f - ((count - 128) / 128.0f);
		}
		gl.glColor4f(0.8f, 0.8f, 0.8f, alpha);
		gl.glDrawArrays(GL11.GL_TRIANGLE_STRIP, 0, 4);
		gl.glPopMatrix();
	}

	int getCount() {
		return count;
	}

	private DancingMissilesScene scene;
	private float scale;
	private int count;
	private int step;
}
