/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL11;

public class ExplosionList extends ActorList {
	public ExplosionList(int aCapacity, DancingMissilesScene aScene) {
		super(aCapacity);

		scene = aScene;
	}

	public void draw(GL11 gl) {
		gl.glDepthMask(false);
		gl.glEnable(GL11.GL_TEXTURE_2D);
		gl.glEnable(GL11.GL_BLEND);
		gl.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE);
		scene.getExplosionTexture().bind(gl);
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		gl.glVertexPointer(3, GL11.GL_FLOAT, 0, vertices.getBuffer());
		gl.glTexCoordPointer(2, GL11.GL_FIXED, 0, texCoordsBuffer);
		for (Actor actor: actors) {
			if (actor != null) {
				Explosion explosion = (Explosion)actor;
				int index = explosion.getCount() / 16;
				int s = ((index & 3) * 32 * 0x10000) / 128;
				int t = ((index / 4) * 32 * 0x10000) / 128;
				texCoords[0] = s;
				texCoords[1] = t;
				texCoords[2] = s + (32 * 0x10000) / 128;
				texCoords[3] = t;
				texCoords[4] = s;
				texCoords[5] = t + (32 * 0x10000) / 128;
				texCoords[6] = s + (32 * 0x10000) / 128;
				texCoords[7] = t + (32 * 0x10000) / 128;
				texCoordsBuffer.position(0);
				texCoordsBuffer.put(texCoords);
				texCoordsBuffer.position(0);
				actor.draw(gl);
			}
		}
		gl.glDisableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		gl.glDisableClientState(GL11.GL_COLOR_ARRAY);
		gl.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glDisable(GL11.GL_BLEND);
		gl.glDisable(GL11.GL_TEXTURE_2D);
		gl.glDepthMask(true);
	}

	private final static float[] VERTICES = {
		-80.0f, -80.0f, 0.0f,
		80.0f, -80.0f, 0.0f,
		-80.0f, 80.0f, 0.0f,
		80.0f, 80.0f, 0.0f,
	};

	private static int[] texCoords = {
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
	};

	private static FloatPoints vertices = new FloatPoints(VERTICES);
	private static IntBuffer texCoordsBuffer;
	static {
		ByteBuffer byteBuffer = ByteBuffer.allocateDirect(texCoords.length * 4);
		byteBuffer.order(ByteOrder.nativeOrder());
		texCoordsBuffer = byteBuffer.asIntBuffer();
	}

	private DancingMissilesScene scene;
}
