/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL11;

import android.content.res.Resources;

class Font {
	public Font(Resources resources) {
		texture = new Texture(resources, R.raw.font);
		ByteBuffer byteBuffer = ByteBuffer.allocateDirect(vertices.length * 4);
		byteBuffer.order(ByteOrder.nativeOrder());
		verticesBuffer = byteBuffer.asIntBuffer();
		byteBuffer = ByteBuffer.allocateDirect(texcoords.length * 4);
		byteBuffer.order(ByteOrder.nativeOrder());
		texcoordsBuffer = byteBuffer.asIntBuffer();
	}

	public void draw(GL11 gl, String string) {
		texture.bind(gl);
		int length = string.length();
		gl.glDisable(GL11.GL_DEPTH_TEST);
		gl.glEnable(GL11.GL_TEXTURE_2D);
		gl.glEnable(GL11.GL_BLEND);
		gl.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE);
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		for (int i = 0; i < length; i++) {
			vertices[0] = (i * 16) * 0x10000;
			vertices[3] = (i * 16 + 16) * 0x10000;
			vertices[6] = (i * 16) * 0x10000;
			vertices[9] = (i * 16 + 16) * 0x10000;
			char characterCode = string.charAt(i);
			int s = ((characterCode & 15) * 16 * 0x10000) / 256;
			int t = ((characterCode / 16) * 16 * 0x10000) / 256;
			texcoords[0] = s;
			texcoords[1] = t;
			texcoords[2] = s + (16 * 0x10000) / 256;
			texcoords[3] = t;
			texcoords[4] = s;
			texcoords[5] = t + (16 * 0x10000) / 256;
			texcoords[6] = s + (16 * 0x10000) / 256;
			texcoords[7] = t + (16 * 0x10000) / 256;
			verticesBuffer.position(0);
			verticesBuffer.put(vertices);
			verticesBuffer.position(0);
			texcoordsBuffer.position(0);
			texcoordsBuffer.put(texcoords);
			texcoordsBuffer.position(0);
			gl.glVertexPointer(3, GL11.GL_FIXED, 0, verticesBuffer);
			gl.glTexCoordPointer(2, GL11.GL_FIXED, 0, texcoordsBuffer);
			gl.glDrawArrays(GL11.GL_TRIANGLE_STRIP, 0, 4);
		}
		gl.glDisable(GL11.GL_BLEND);
		gl.glDisable(GL11.GL_TEXTURE_2D);
		gl.glEnable(GL11.GL_DEPTH_TEST);
	}

	private static int[] vertices = {
		0 * 0x10000, 0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 16 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 16 * 0x10000, 0 * 0x10000,
	};

	private static int[] texcoords = {
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
		0 * 0x10000, 0 * 0x10000,
	};

	private Texture texture;
	private IntBuffer verticesBuffer;
	private IntBuffer texcoordsBuffer;
}
