/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

public class Matrix {
	public Matrix() {
		super();

		values = new float[16];
		for (int i = 0;i < values.length;i++) {
			values[i] = 0.0f;
		}
		values[0 * 4 + 0] = 1.0f;
		values[1 * 4 + 1] = 1.0f;
		values[2 * 4 + 2] = 1.0f;
		values[3 * 4 + 3] = 1.0f;
	}

	Matrix getInverted() {
		Matrix result = new Matrix();
		for(int i = 0;i < 4;i++)
		{
			for(int j = 0;j < 4;j++)
			{
				result.values[i * 4 + j] = values[j * 4 + i];
			}
		}

		return result;
	}
	
	public Matrix getMultiplied(Matrix matrix) {
		Matrix result = new Matrix();
		for(int i = 0;i < 4;i++)
		{
			for(int j = 0;j < 4;j++)
			{
				float value = 0.0f;
				for(int k = 0;k < 4;k++)
				{
					value += values[i * 4 + k] * matrix.values[k * 4 + j];
				}
				result.values[i * 4 + j] = value;
			}
		}

		return result;
	}

	public Vector getMultipliedVector(Vector vector)
	{
		Vector result = new Vector();
		result.setX(values[0 * 4 + 0] * vector.getX() + values[0 * 4 + 1] * vector.getY() + values[0 * 4 + 2] * vector.getZ());
		result.setY(values[1 * 4 + 0] * vector.getX() + values[1 * 4 + 1] * vector.getY() + values[1 * 4 + 2] * vector.getZ());
		result.setZ(values[2 * 4 + 0] * vector.getX() + values[2 * 4 + 1] * vector.getY() + values[2 * 4 + 2] * vector.getZ());

		return result;
	}
	
	public float getAt(int index) {
		return values[index];
	}

	public void setAt(int index, float value) {
		values[index] = value;
	}

	public float[] getValues() {
		return values;
	}

	public static Matrix getRotation(Vector vector, float angle) {
		float x = vector.getX();
		float y = vector.getY();
		float z = vector.getZ();
		float length = (float)Math.sqrt(x * x + y * y + z * z);
		if(length <= 0.000001f)
		{
			// Avoid zero division
			return new Matrix();
		}

		x /= length;
		y /= length;
		z /= length;
		float sine = (float)Math.sin(angle * Math.PI / 180.0f);
		float cosine = (float)Math.cos(angle * Math.PI / 180.0f);
		float onemcosine = 1.0f - cosine;
		float xx = onemcosine * x * x;
		float yy = onemcosine * y * y;
		float zz = onemcosine * z * z;
		float xy = onemcosine * x * y;
		float yz = onemcosine * y * z;
		float zx = onemcosine * z * x;
		float xs = x * sine;
		float ys = y * sine;
		float zs = z * sine;

		Matrix result = new Matrix();
		result.setAt(0, xx + cosine);
		result.setAt(1, xy + zs);
		result.setAt(2, zx - ys);
		result.setAt(3, 0.0f);
		result.setAt(4, xy - zs);
		result.setAt(5, yy + cosine);
		result.setAt(6, yz + xs);
		result.setAt(7, 0.0f);
		result.setAt(8, zx + ys);
		result.setAt(9, yz - xs);
		result.setAt(10, zz + cosine);
		result.setAt(11, 0.0f);
		result.setAt(12, 0.0f);
		result.setAt(13, 0.0f);
		result.setAt(14, 0.0f);
		result.setAt(15, 1.0f);

		return result;
	}

	private float values[];
}
