/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.util.Random;

import javax.microedition.khronos.opengles.GL11;

public class Missile extends Actor {
	public Missile(DancingMissilesScene aScene, int aSmokeInterval) {
		super();

		scene = aScene;
		Random random = new Random();
		turningSpeed = (float)(random.nextInt(256) + 384.0f) / 128.0f;
		count = random.nextInt(1200) + 1200;
		smokeInterval = aSmokeInterval;
		smokeCount = 0;
		offsetCount = random.nextInt(180) + 180;
		position = Vector.getRandomizedInCube(-2560.0f, 2560.0f, -2560.0f, 2560.0f, -2560.0f, 2560.0f);
		velocity = Vector.getRandomizedInCube(-2.5f, 2.5f, -2.5f, 2.5f, -2.5f, 2.5f);
		targetOffset = Vector.getRandomizedInCube(-64.0f, 64.0f, -64.0f, 64.0f, -64.0f, 64.0f);
	}

	@Override
	public void process() {
		Vector targetPosition = scene.getTarget().getPosition();
		Vector targetVelocity = new Vector(scene.getTarget().getVelocity());
		targetVelocity.multiply(10.0f);
		targetPosition = targetPosition.getAdded(targetOffset);
		targetPosition.add(targetVelocity);
		Vector distance = targetPosition.getSubtracted(position);
		Vector invertedDistance = direction.getMultipliedVector(distance);
		float roll = (float)(Math.atan2(invertedDistance.getX(), -invertedDistance.getY()) * 180.0f / Math.PI);
		if (roll > turningSpeed) {
			roll = turningSpeed;
		}
		if (roll < -turningSpeed) {
			roll = -turningSpeed;
		}
		Vector axis = new Vector(direction.getAt(2 * 4 + 0), direction.getAt(2 * 4 + 1), direction.getAt(2 * 4 + 2));
		Matrix rotation = Matrix.getRotation(axis, roll);
		direction = direction.getMultiplied(rotation);
		invertedDistance = direction.getMultipliedVector(distance);
		float pitch = (float)(Math.atan2(-invertedDistance.getY(), invertedDistance.getZ()) * 180.0f / Math.PI);
		if (pitch > turningSpeed) {
			pitch = turningSpeed;
		}
		if (pitch < -turningSpeed) {
			pitch = -turningSpeed;
		}
		axis.set(direction.getAt(0 * 4 + 0), direction.getAt(0 * 4 + 1), direction.getAt(0 * 4 + 2));
		rotation = Matrix.getRotation(axis, pitch);
		direction = direction.getMultiplied(rotation);

		Vector directionVector = new Vector(direction.getAt(2 * 4 + 0), direction.getAt(2 * 4 + 1), direction.getAt(2 * 4 + 2));
		velocity.setX(velocity.getX() * 0.91f + directionVector.getX() * 1.0f);
		velocity.setY(velocity.getY() * 0.91f + directionVector.getY() * 1.0f);
		velocity.setZ(velocity.getZ() * 0.91f + directionVector.getZ() * 1.0f);
		position.add(velocity);

		Random random = new Random();
		Vector aPosition = new Vector(position.getX() - directionVector.getX() * 12.0f, position.getY() - directionVector.getY() * 12.0f, position.getZ() - directionVector.getZ() * 12.0f);
		Vector randomVelocity = Vector.getRandomized(0.6f);
		Vector aVelocity = new Vector(directionVector.getX() * -0.5f + randomVelocity.getX(), directionVector.getY() * -0.5f + randomVelocity.getY(), directionVector.getZ() * -0.5f + randomVelocity.getZ());
		scene.getExplosions().add(new Explosion(aPosition, aVelocity, 0.17f, 72, scene));
		smokeCount++;
		if (smokeCount >= smokeInterval) {
			aPosition.set(position.getX() - directionVector.getX() * 20.0f, position.getY() - directionVector.getY() * 20.0f, position.getZ() - directionVector.getZ() * 20.0f);
			scene.getSmokes().add(new Smoke(aPosition, aVelocity, scene));

			smokeCount = 0;
		}

		offsetCount--;
		if (offsetCount <= 0) {
			offsetCount = random.nextInt(180) + 180;
			targetOffset = Vector.getRandomizedInCube(-64.0f, 64.0f, -64.0f, 64.0f, -64.0f, 64.0f);
		}

		count--;
		if (count <= 0) {
			for (int i = 0;i < 5;i++) {
				Vector explosionPosition = new Vector(position.getX(), position.getY(), position.getZ());
				Vector explosionVelocity = new Vector(velocity);
				explosionVelocity.add(Vector.getRandomized(5.0f));
				scene.getExplosions().add(new Explosion(explosionPosition, explosionVelocity, 1.0f, 4, scene));
			}
			scene.getMissiles().remove(this);
		}
	}

	@Override
	public void draw(GL11 gl) {
		gl.glEnable(GL11.GL_LIGHTING);
		gl.glEnable(GL11.GL_LIGHT0);
		gl.glEnable(GL11.GL_TEXTURE_2D);
		scene.getMissileTexture().bind(gl);
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL11.GL_COLOR_ARRAY);
		gl.glEnableClientState(GL11.GL_NORMAL_ARRAY);
		gl.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		gl.glVertexPointer(3, GL11.GL_FLOAT, 0, vertices.getBuffer());
		gl.glColorPointer(4, GL11.GL_FLOAT, 0, colors.getBuffer());
		gl.glNormalPointer(GL11.GL_FLOAT, 0, normals.getBuffer());
		gl.glTexCoordPointer(2, GL11.GL_FLOAT, 0, texCoords.getBuffer());
		gl.glPushMatrix();
		gl.glTranslatef(position.getX(), position.getY(), position.getZ());
		gl.glMultMatrixf(direction.getValues(), 0);
		gl.glDrawArrays(GL11.GL_TRIANGLES, 0, 42);
		gl.glPopMatrix();
		gl.glDisableClientState(GL11.GL_NORMAL_ARRAY);
		gl.glDisableClientState(GL11.GL_COLOR_ARRAY);
		gl.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glDisable(GL11.GL_TEXTURE_2D);
		gl.glDisable(GL11.GL_LIGHT0);
		gl.glDisable(GL11.GL_LIGHTING);
	}

	private final static float[] VERTICES = {
		-2.5f, 2.5f, -10.0f,
		2.5f, 2.5f, -10.0f,
		-2.5f, 2.5f, 10.0f,

		2.5f, 2.5f, 10.0f,
		2.5f, 2.5f, -10.0f,
		-2.5f, 2.5f, 10.0f,

		-2.5f, -2.5f, -10.0f,
		2.5f, -2.5f, -10.0f,
		-2.5f, -2.5f, 10.0f,

		2.5f, -2.5f, 10.0f,
		2.5f, -2.5f, -10.0f,
		-2.5f, -2.5f, 10.0f,

		2.5f, -2.5f, -10.0f,
		2.5f, 2.5f, -10.0f,
		2.5f, -2.5f, 10.0f,

		2.5f, 2.5f, 10.0f,
		2.5f, 2.5f, -10.0f,
		2.5f, -2.5f, 10.0f,

		-2.5f, -2.5f, -10.0f,
		-2.5f, 2.5f, -10.0f,
		-2.5f, -2.5f, 10.0f,

		-2.5f, 2.5f, 10.0f,
		-2.5f, 2.5f, -10.0f,
		-2.5f, -2.5f, 10.0f,

		-2.5f, -2.5f, -10.0f,
		2.5f, -2.5f, -10.0f,
		-2.5f, 2.5f, -10.0f,

		2.5f, -2.5f, -10.0f,
		2.5f, 2.5f, -10.0f,
		-2.5f, 2.5f, -10.0f,

		-2.5f, -2.5f, 10.0f,
		2.5f, -2.5f, 10.0f,
		-2.5f, 2.5f, 10.0f,

		2.5f, -2.5f, 10.0f,
		2.5f, 2.5f, 10.0f,
		-2.5f, 2.5f, 10.0f,

		0.0f, 0.0f, -2.5f,
		7.0f, 0.0f, -9.5f,
		-7.0f, 0.0f, -9.5f,

		0.0f, 0.0f, -2.5f,
		0.0f, 7.0f, -9.5f,
		0.0f, -7.0f, -9.5f,
	};

	private final static float[] TEX_COORDS = {
		0.0f, 0.0f,
		0.0f, 0.1875f,
		1.0f, 0.0f,
		
		1.0f, 0.1875f,
		0.0f, 0.1875f,
		1.0f, 0.0f,
		
		0.0f, 0.0f,
		0.0f, 0.375f,
		1.0f, 0.0f,

		1.0f, 0.375f,
		0.0f, 0.375f,
		1.0f, 0.0f,

		0.0f, 0.0f,
		0.0f, 0.5625f,
		1.0f, 0.0f,
		
		1.0f, 0.5625f,
		0.0f, 0.5625f,
		1.0f, 0.0f,

		0.0f, 0.0f,
		0.0f, 0.5f,
		1.0f, 0.0f,

		1.0f, 0.5f,
		0.0f, 0.5f,
		1.0f, 0.0f,

		0.0f, 0.0f,
		0.0f, 0.1875f,
		0.0f, 0.5625f,

		0.0f, 0.1875f,
		0.0f, 0.375f,
		0.0f, 0.5625f,
		
		1.0f, 0.0f,
		1.0f, 0.1875f,
		1.0f, 0.5625f,

		1.0f, 0.1875f,
		1.0f, 0.375f,
		1.0f, 0.5625f,

		0.5f, 0.75f,
		0.0f, 1.0f,
		1.0f, 1.0f,
		
		0.5f, 0.75f,
		0.0f, 1.0f,
		1.0f, 1.0f,
	};
	
	private final static float[] COLORS = {
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,

		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
	};

	private final static float[] NORMALS = {
		-0.707f, 0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		0.707f, 0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		-0.707f, -0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,
		-0.707f, -0.707f, 0.0f,

		0.707f, -0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,
		-0.707f, -0.707f,0.0f,

		0.707f, -0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,

		0.707f, 0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,

		-0.707f, -0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,
		-0.707f, -0.707f, 0.0f,

		-0.707f, 0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,
		-0.707f, -0.707f, 0.0f,

		-0.707f, -0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		0.707f, -0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		-0.707f, -0.707f, 0.0f,
		0.707f, -0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		0.707f, -0.707f, 0.0f,
		0.707f, 0.707f, 0.0f,
		-0.707f, 0.707f, 0.0f,

		0.0f, 1.0f, 0.0f,
		0.0f, 1.0f, 0.0f,
		0.0f, 1.0f, 0.0f,

		1.0f, 0.0f, 0.0f,
		1.0f, 0.0f, 0.0f,
		1.0f, 0.0f, 0.0f,
	};

	private static FloatPoints vertices = new FloatPoints(VERTICES);
	private static FloatPoints colors = new FloatPoints(COLORS);
	private static FloatPoints normals = new FloatPoints(NORMALS);
	private static FloatPoints texCoords = new FloatPoints(TEX_COORDS);

	private Vector targetOffset;
	private DancingMissilesScene scene;
	private float turningSpeed;
	private int count;
	private int smokeInterval;
	private int smokeCount;
	private int offsetCount;
}
