/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import android.content.SharedPreferences;

public class Settings {
	public static Settings getInstance() {
		if (instance != null) {
			return instance;
		}

		instance = new Settings();

		return instance;
	}

	public void update(SharedPreferences preferences) {
		String missileNumString = preferences.getString("missile_num", "4");
		missileNum = Integer.valueOf(missileNumString).intValue();
		String smokeIntervalString = preferences.getString("smoke_interval", "2");
		smokeInterval = Integer.valueOf(smokeIntervalString).intValue();
		String smokeAlphaString = preferences.getString("smoke_alpha", "0.2");
		smokeAlpha = Float.valueOf(smokeAlphaString).floatValue();
		String backgroundTypeString = preferences.getString("background_type", "0");
		backgroundType = Integer.valueOf(backgroundTypeString).intValue();
		targetShown = preferences.getBoolean("target_shown", false);
		informationShown = preferences.getBoolean("information_shown", false);
	}

	public int getMissileNum() {
		return missileNum;
	}

	public int getSmokeInterval() {
		return smokeInterval;
	}

	public float getSmokeAlpha() {
		return smokeAlpha;
	}

	public int getBackgroundType() {
		return backgroundType;
	}

	public boolean getTargetShown() {
		return targetShown;
	}

	public boolean getInformationShown() {
		return informationShown;
	}

	private Settings() {
		super();

		instance = this;

		missileNum = 4;
		smokeInterval = 2;
		smokeAlpha = 0.2f;
		backgroundType = 0;
		targetShown = false;
		informationShown = false;
	}

	public static final int BACKGROUND_SEA = 0;
	public static final int BACKGROUND_FOREST = 1;
	public static final int BACKGROUND_DESERT = 2;

	private static Settings instance = null;

	private int missileNum;
	private int smokeInterval;
	private float smokeAlpha;
	private int backgroundType;
	private boolean targetShown;
	private boolean informationShown;
}
