/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.opengles.GL11;

public class Smoke extends Actor {
	public Smoke(Vector aPosition, Vector aVelocity, DancingMissilesScene aScene) {
		super();

		position.set(aPosition);
		velocity.set(aVelocity);
		scene = aScene;
		count = 0;
		step = 4;
	}

	@Override
	public void process() {
		super.process();
		count += step;
		if (count >= 256) {
			scene.getSmokes().remove(this);			
		}
	}

	@Override
	public void draw(GL11 gl) {
		gl.glPushMatrix();
		gl.glTranslatef(position.getX(), position.getY(), position.getZ());
		gl.glMultMatrixf(scene.getCamera().getDirection().getValues(), 0);
		float baseAlpha = Settings.getInstance().getSmokeAlpha();
		float alpha = 1.0f;
		if (count >= 192)
		{
			alpha = 1.0f - ((count - 192) / 64.0f);				
		}
		gl.glColor4f(1.0f, 1.0f, 1.0f, alpha * baseAlpha);
		gl.glDrawArrays(GL11.GL_TRIANGLE_STRIP, 0, 4);
		gl.glPopMatrix();
	}

	int getCount() {
		return count;
	}

	private DancingMissilesScene scene;
	private int count;
	private int step;
}
