/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL11;

public class SpeedLines extends Actor {
	public SpeedLines(DancingMissilesScene aScene) {
		super();

		linePositions = new Vector[LINE_NUM];
		for (int i = 0;i < LINE_NUM;i++) {
			linePositions[i] = Vector.getRandomizedInCube(-256.0f, 256.0f, -256.0f, 256.0f, -256.0f, 256.0f);
		}
		scene = aScene;
		position.set(scene.getCamera().getPosition());
	}

	@Override
	public void process() {
		position.set(scene.getCamera().getPosition());
		for (int i = 0;i < LINE_NUM;i++) {
			Vector linePosition = linePositions[i];
			float diffX = linePosition.getX() - position.getX();
			if (diffX > 256.0f) {
				linePosition.setX(linePosition.getX() - 512.0f);
			}
			if (diffX < -256.0f) {
				linePosition.setX(linePosition.getX() + 512.0f);
			}
			float diffY = linePosition.getY() - position.getY();
			if (diffY > 256.0f) {
				linePosition.setY(linePosition.getY() - 512.0f);
			}
			if (diffY < -256.0f) {
				linePosition.setY(linePosition.getY() + 512.0f);
			}
			float diffZ = linePosition.getZ() - position.getZ();
			if (diffZ > 256.0f) {
				linePosition.setZ(linePosition.getZ() - 512.0f);
			}
			if (diffZ < -256.0f) {
				linePosition.setZ(linePosition.getZ() + 512.0f);
			}
		}
	}

	@Override
	public void draw(GL11 gl) {
		Vector cameraVelocity = scene.getCamera().getVelocity();
		for (int i = 0;i < LINE_NUM;i++) {
			Vector linePosition = linePositions[i];
			vertices[i * 6] = (int)(linePosition.getX() * 0x10000);
			vertices[i * 6 + 1] = (int)(linePosition.getY() * 0x10000);
			vertices[i * 6 + 2] = (int)(linePosition.getZ() * 0x10000);
			vertices[i * 6 + 3] = (int)((linePosition.getX() + cameraVelocity.getX() * 10.0f) * 0x10000);
			vertices[i * 6 + 4] = (int)((linePosition.getY() + cameraVelocity.getY() * 10.0f) * 0x10000);
			vertices[i * 6 + 5] = (int)((linePosition.getZ() + cameraVelocity.getZ() * 10.0f) * 0x10000);
		}
		verticesBuffer.position(0);
		verticesBuffer.put(vertices);
		verticesBuffer.position(0);
		colorsBuffer.position(0);
		colorsBuffer.put(colors);
		colorsBuffer.position(0);
	
		gl.glDepthMask(false);
		gl.glEnable(GL11.GL_BLEND);
		gl.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE);
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL11.GL_COLOR_ARRAY);
		gl.glVertexPointer(3, GL11.GL_FIXED, 0, verticesBuffer);
		gl.glColorPointer(4, GL11.GL_FIXED, 0, colorsBuffer);
		gl.glDrawArrays(GL11.GL_LINES, 0, LINE_NUM * 2);
		gl.glDisableClientState(GL11.GL_COLOR_ARRAY);
		gl.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glDisable(GL11.GL_BLEND);
		gl.glDepthMask(true);
	}

	private final static int LINE_NUM = 64;

	private static int[] vertices = new int[LINE_NUM * 3 * 2];
	private static int[] colors;
	static {
		colors = new int[LINE_NUM * 4 * 2];
		for (int i = 0;i < LINE_NUM;i++) {
			colors[i * 8] = 256 * 0x100;
			colors[i * 8 + 1] = 256 * 0x100;
			colors[i * 8 + 2] = 256 * 0x100;
			colors[i * 8 + 3] = 48 * 0x100;
			colors[i * 8 + 4] = 256 * 0x100;
			colors[i * 8 + 5] = 256 * 0x100;
			colors[i * 8 + 6] = 256 * 0x100;
			colors[i * 8 + 7] = 0 * 0x100;
		}
	}
	private static IntBuffer verticesBuffer;
	static {
		ByteBuffer byteBuffer = ByteBuffer.allocateDirect(vertices.length * 4);
		byteBuffer.order(ByteOrder.nativeOrder());
		verticesBuffer = byteBuffer.asIntBuffer();
	}
	private static IntBuffer colorsBuffer;
	static {
		ByteBuffer byteBuffer = ByteBuffer.allocateDirect(colors.length * 4);
		byteBuffer.order(ByteOrder.nativeOrder());
		colorsBuffer = byteBuffer.asIntBuffer();
	}

	private DancingMissilesScene scene;
	private Vector[] linePositions;
}
