/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.opengles.GL11;

public class Target extends Actor {
	public Target() {
		super();

		targetPosition = Vector.getRandomizedInCube(-256.0f, 256.0f, -256.0f, 256.0f, -256.0f, 256.0f);
		count = 300;
		offset = Vector.getRandomizedInCube(-128.0f, 128.0f, -128.0f, 128.0f, -128.0f, 128.0f);
		offsetCount = 60;
	}

	@Override
	public void process() {
		count--;
		if (count <= 0) {
			targetPosition = Vector.getRandomizedInCube(-2560.0f, 2560.0f, -2560.0f, 2560.0f, -2560.0f, 2560.0f);
			count = 300;
		}
		offsetCount--;
		if (offsetCount <= 0) {
			offset = Vector.getRandomizedInCube(-32.0f, 32.0f, -32.0f, 32.0f, -32.0f, 32.0f);
			offsetCount = 60;
		}

		Vector movingTarget = targetPosition.getAdded(offset);
		Vector distance = movingTarget.getSubtracted(position);
		Vector invertedDistance = direction.getMultipliedVector(distance);
		float roll = (float)(Math.atan2(invertedDistance.getX(), -invertedDistance.getY()) * 180.0f / Math.PI) * 0.05f;
		if (roll > 3.0f) {
			roll = 3.0f;
		}
		if (roll < -3.0f) {
			roll = -3.0f;
		}
		Vector axis = new Vector(direction.getAt(2 * 4 + 0), direction.getAt(2 * 4 + 1), direction.getAt(2 * 4 + 2));
		Matrix rotation = Matrix.getRotation(axis, roll);
		direction = direction.getMultiplied(rotation);
		invertedDistance = direction.getMultipliedVector(distance);
		float pitch = (float)(Math.atan2(-invertedDistance.getY(), invertedDistance.getZ()) * 180.0f / Math.PI) * 0.05f;
		if (pitch > 2.0f) {
			pitch = 2.0f;
		}
		if (pitch < -2.0f) {
			pitch = -2.0f;
		}
		axis.set(direction.getAt(0 * 4 + 0), direction.getAt(0 * 4 + 1), direction.getAt(0 * 4 + 2));
		rotation = Matrix.getRotation(axis, pitch);
		direction = direction.getMultiplied(rotation);

		Vector directionVector = new Vector(direction.getAt(2 * 4 + 0), direction.getAt(2 * 4 + 1), direction.getAt(2 * 4 + 2));
		velocity.setX(velocity.getX() * 0.91f + directionVector.getX() * 0.82f);
		velocity.setY(velocity.getY() * 0.91f + directionVector.getY() * 0.82f - 0.01f);
		velocity.setZ(velocity.getZ() * 0.91f + directionVector.getZ() * 0.82f);
		position.add(velocity);
	}

	@Override
	public void draw(GL11 gl) {
		gl.glDepthMask(false);
		gl.glEnable(GL11.GL_BLEND);
		gl.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE);
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glVertexPointer(3, GL11.GL_FLOAT, 0, vertices.getBuffer());
		gl.glColor4f(0.1f, 0.25f, 1.0f, 1.0f);
		gl.glPushMatrix();
		gl.glTranslatef(position.getX(), position.getY(), position.getZ());
		gl.glMultMatrixf(direction.getValues(), 0);
		gl.glDrawArrays(GL11.GL_LINES, 0, 12);
		gl.glPopMatrix();
		gl.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glDisable(GL11.GL_BLEND);
		gl.glDepthMask(true);
	}

	private final static float[] VERTICES = {
		-20.0f, 0.0f, -30.0f,
		0.0f, 0.0f, 30.0f,
		0.0f, 0.0f, 30.0f,
		20.0f, 0.0f, -30.0f,
		20.0f, 0.0f, -30.0f,
		-20.0f, 0.0f, -30.0f,
		0.0f, 0.0f, 30.0f,
		0.0f, 20.0f, -30.0f,
		0.0f, 20.0f, -30.0f,
		0.0f, 0.0f, -30.0f,
		0.0f, 0.0f, -30.0f,
		0.0f, 0.0f, 30.0f,
	};

	private static FloatPoints vertices = new FloatPoints(VERTICES);

	private Vector targetPosition;
	private Vector offset;
	private int count;
	private int offsetCount;
}
