/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.io.IOException;
import java.io.InputStream;

import javax.microedition.khronos.opengles.GL11;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLUtils;

public class Texture {
	public Texture(Resources aResources, int aId) {
		textures = new int[1];
		resources = aResources;
		id = aId;
		loaded = false;
	}

	public void bind(GL11 gl) {
		if (!loaded) {
			load(gl);
		}
		else {
			gl.glBindTexture(GL11.GL_TEXTURE_2D, textures[0]);
		}
	}
	
	public void load(GL11 gl) {
		gl.glGenTextures(1, textures, 0);
		gl.glBindTexture(GL11.GL_TEXTURE_2D, textures[0]);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_GENERATE_MIPMAP, GL11.GL_TRUE);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR_MIPMAP_NEAREST);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
		gl.glTexEnvx(GL11.GL_TEXTURE_ENV, GL11.GL_TEXTURE_ENV_MODE, GL11.GL_MODULATE);
		InputStream stream = resources.openRawResource(id);
		Bitmap bitmap;
		try {
			bitmap = BitmapFactory.decodeStream(stream);
		}
		finally {
			try {
				stream.close();
			}
			catch (IOException exception) {
				// Do nothing
			}
		}
		GLUtils.texImage2D(GL11.GL_TEXTURE_2D, 0, bitmap, 0);
		bitmap.recycle();
		loaded = true;
	}

	public void delete(GL11 gl) {
		gl.glDeleteTextures(1, textures, 0);
		loaded = false;
	}

	public void invalidate(GL11 gl) {
		delete(gl);
	}

	private Resources resources;
	private int[] textures;
	private int id;
	private boolean loaded;
}
