/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import java.util.Random;

public class Vector {
	public Vector() {
		if (random == null) {
			random = new Random();
		}

		x = 0.0f;
		y = 0.0f;
		z = 0.0f;
	}

	public Vector(float aX, float aY, float aZ) {
		set(aX, aY, aZ);
	}

	public Vector(Vector vector) {
		set(vector);
	}

	public void add(Vector vector) {
		x += vector.x;
		y += vector.y;
		z += vector.z;
	}

	public Vector getAdded(Vector vector) {
		Vector result = new Vector();
		result.x = x + vector.x;
		result.y = y + vector.y;
		result.z = z + vector.z;

		return result;
	}

	public void multiply(float value) {
		x *= value;
		y *= value;
		z *= value;
	}

	public Vector getSubtracted(Vector vector) {
		Vector result = new Vector();
		result.x = x - vector.x;
		result.y = y - vector.y;
		result.z = z - vector.z;

		return result;
	}

	public Vector getCrossProduct(Vector vector) {
		Vector result = new Vector();
		result.x = y * vector.z - z * vector.y;
		result.y = z * vector.x - x * vector.z;
		result.z = x * vector.y - y * vector.x;

		return result;
	}

	public void normalize() {
		float length = (float)Math.sqrt(x * x + y * y + z * z);
		x /= length;
		y /= length;
		z /= length;
	}

	public float getLength() {
		return (float)Math.sqrt(x * x + y * y + z * z);		
	}

	public float getX() {
		return x;
	}

	public void setX(float aX) {
		x = aX;
	}

	public float getY() {
		return y;
	}

	public void setY(float aY) {
		y = aY;
	}

	public float getZ() {
		return z;
	}

	public void setZ(float aZ) {
		z = aZ;
	}
	
	public void set(float aX, float aY, float aZ) {
		x = aX;
		y = aY;
		z = aZ;
	}

	public void set(Vector vector) {
		x = vector.x;
		y = vector.y;
		z = vector.z;
	}

	public static Vector getRandomized(float maximumLength) {
		float x = random.nextFloat() - 0.5f;
		float y = random.nextFloat() - 0.5f;
		float z = random.nextFloat() - 0.5f;
		float length = (float)Math.sqrt(x * x + y * y + z * z);
		float newLength = random.nextFloat() * maximumLength;
		Vector result = new Vector();
		result.set(x * newLength / length, y * newLength / length, z * newLength / length);

		return result;
	}

	public static Vector getRandomizedInCube(float minimumX, float maximumX, float minimumY, float maximumY, float minimumZ, float maximumZ) {
		float x = random.nextFloat() * (maximumX - minimumX) + minimumX;
		float y = random.nextFloat() * (maximumY - minimumY) + minimumY;
		float z = random.nextFloat() * (maximumZ - minimumZ) + minimumZ;
		Vector result = new Vector();
		result.set(x, y, z);

		return result;
	}

	private static Random random;

	private float x;
	private float y;
	private float z;
}
